from pathlib import Path
from navalmartin_mir_vision_utils import load_img, ImageLoadersEnumType

from navalmartin_mir_vision_utils.image_quality import brisque

if __name__ == '__main__':
    image_path = Path("/home/alex/qi3/mir-engine/datasets/cracks_v_3_id_8/train/cracked/img_9_9.jpg")
    image = load_img(path=image_path, loader=ImageLoadersEnumType.PIL)
    brisque_score = brisque.score(image)
    print(brisque_score)


from typing import List, Dict
import numpy as np
from PIL import Image
from PIL.ImageStat import Stat
from scipy.stats import norm as normal_distribution
from scipy.stats import entropy
from scipy.special import rel_entr
from math import log2


def gamma_correction(img: np.ndarray, gamma_val) -> np.ndarray:
    """Implements gamma correction on the given image

    Parameters
    ----------
    img: The image to perform the correction
    gamma_val: The gamma value

    Returns
    -------

    """

    return np.power(img, gamma_val)


def compute_image_statistics(image: Image) -> Stat:
    return Stat(image_or_list=image)


def compute_image_channels_means(image: Image) -> List[float]:

    stats = Stat(image_or_list=image)
    return stats.mean


def compute_image_channels_variance(image: Image) -> List[float]:
    stats = Stat(image_or_list=image)
    return stats.var


def compute_image_channels_median(image: Image) -> List[float]:
    stats = Stat(image_or_list=image)
    return stats.median


def fit_gaussian_distribution_on_image(image: Image) -> List[Dict]:
    """Fits a gaussian distribution on every channel of the
    image. Returns a list of tuples the mean and standard deviation of the resulting
    distribution

    Parameters
    ----------
    image: The image to fit

    Returns
    -------

    """
    # see also this post:
    # https://stackoverflow.com/questions/52962969/number-of-channels-in-pil-pillow-image?rq=1
    channels = image.getbands()

    if len(channels) == 3:

        # split the channels
        r, g, b = image.split()

        red_channel = fit_gaussian_distribution(data=r)
        green_channel = fit_gaussian_distribution(data=g)
        blue_channel = fit_gaussian_distribution(data=b)

        return [{'green': green_channel, 'red': red_channel, 'blue': blue_channel}]
    else:
        raise NotImplementedError("For len(channels) != 3 "
                                  "fit_gaussian_distribution_on_image is not implemented")


def fit_gaussian_distribution(data: List[float]) -> List[float]:
    """Fits a Gaussian distribution on the given data and
    returns the mean and standard deviation of the resulting
    distribution

    Parameters
    ----------
    data: The data to fit

    Returns
    -------

    A list containing the mean and standard deviation of
    the fitted distribution
    """
    return normal_distribution.fit(data)


def kl_divergence(q: List[float], p: List[float],
                  do_test_sum: bool = True) -> float:
    """Compute the KL divergence between the two given mass functions
    Note that the elements in each array must sum to one.
    The SciPy library provides the kl_div() function for calculating the KL divergence,
    although with a different definition as defined here.
    It also provides the rel_entr() function for calculating the relative entropy,
    which matches the definition of KL divergence here.
    This is odd as “relative entropy” is often used as a synonym for “KL divergence.”

    Parameters
    ----------
    q: The first mass function
    p: The second mass function
    do_test_sum: Test if the elements in the two arrays
    sum to one
    Returns
    -------

    The difference between the two mass functions
    """

    if do_test_sum:
        assert sum(q) == 1 and sum(p) == 1

    return rel_entr(q, p)


def jensen_shannon_distance(q: List[float], p: List[float],
                            do_test_sum: bool = True) -> float:
    """Calculate the Jensen-Shannon divergence
    The JS divergence quantifies the difference (or similarity) between two probability distributions.
    Note that the elements in q and p should sum up to one.

    Parameters
    ----------
    q: The first mass function
    p: The second mass function
    do_test_sum: Test if the elements in the two arrays
    sum to one

    Returns
    -------

    """

    if do_test_sum:
        assert sum(q) == 1 and sum(p) == 1

    # calculate m
    m = (p + q) / 2.

    # compute Jensen Shannon Divergence
    divergence = (entropy(p, m) + entropy(q, m)) / 2

    # compute the Jensen Shannon Distance
    distance = np.sqrt(divergence)
    return distance



import unittest
from pathlib import Path

from src.navalmartin_mir_vision_utils.image_enums import ImageLoadersEnumType
from src.navalmartin_mir_vision_utils.image_utils import load_img
from src.navalmartin_mir_vision_utils.image_quality.brisque import score


class TestBrisqueImageQuality(unittest.TestCase):

    def test_get_img_files(self):
        image = load_img(path=Path('/home/alex/qi3/mir_vision_utils/test_data/img_18_3.jpg'),
                         loader=ImageLoadersEnumType.PIL,
                         transformer=None)

        score(image=image)


if __name__ == '__main__':
    unittest.main()



from pathlib import Path
from navalmartin_mir_vision_utils import load_img, ImageLoadersEnumType
from navalmartin_mir_vision_utils.statistics import compute_image_statistics, fit_gaussian_distribution_on_image

if __name__ == '__main__':

    image_path = Path("/home/alex/qi3/mir-engine/datasets/cracks_v_3_id_8/train/cracked/img_9_9.jpg")
    image = load_img(path=image_path, loader=ImageLoadersEnumType.PIL)

    print(f"Image size {image.size}")
    print(f"Image bands {image.getbands()}")

    image_stats = compute_image_statistics(image)
    print(f"Image channel mean {image_stats.mean}")
    print(f"Image channel var {image_stats.var}")
    print(f"Image channel median {image_stats.median}")

    channels_fit = fit_gaussian_distribution_on_image(image=image)
    print(f"Gaussian distribution channel fit: {channels_fit}")





class InvalidPILImageMode(Exception):
    def __init__(self, pil_mode: str):
        self.message = f"The given PIL mode {pil_mode} is invalid"

    def __str__(self):
        return self.message


class InvalidConfiguration(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self) -> str:
        return self.message


class InvalidImageFile(Exception):
    def __init__(self, filename: str):
        self.message = f"File {filename} is not a valid image file"

    def __str__(self) -> str:
        return self.message



from pathlib import Path
from navalmartin_mir_vision_utils import (is_valid_pil_image_file,
                                          get_pil_image_size,
                                          get_img_files,
                                          pil_image_to_bytes_string)

from navalmartin_mir_vision_utils.mir_vison_io import get_md5_checksum

if __name__ == '__main__':

    image_file = Path("/home/alex/qi3/mir-engine/datasets/cracks_v_3_id_8/train/cracked/img_9_9.jpg")
    image = is_valid_pil_image_file(image=image_file)
    if image is not None:
        print("The provided image is OK")
        image_size = get_pil_image_size(image=image)
        print(f"Image size is {image_size}")
    else:
        print("The provided image is NOT OK")

    base_path = Path("/home/alex/qi3/mir-engine/datasets/cracks_v_3_id_8/train/cracked/")
    image_files = get_img_files(base_path=base_path)
    print(f"There are {len(image_files)} in {base_path}")

    # calculate file checksum
    image_checksum = get_md5_checksum(file=image_file)
    print(f"Calculated MD5 checksum {image_checksum}")

    image_checksum = get_md5_checksum(file=image.tobytes())
    print(f"Calculated MD5 checksum {image_checksum}")



from setuptools import setup

setup(
    name='navalmartin_mir_vision_utils',
    packages=['navalmartin_mir_vision_utils'],
    version='0.0.1',
    description='Various utilities for working with images in the mir project.',
    author='Alexandros Giavaras',
    author_email='a.giavaras@gmail.com',
    url='https://github.com/Navalmartin/navalmartin_mir_vision_utils',
    download_url='https://github.com/pockerman/awsutils/awsutils-0.0.3.tar.gz',
    keywords=['images', 'computer-vision'],
    classifiers=[],
    scripts=[],
)



import matplotlib.pyplot as plt

from navalmartin_mir_vision_utils import WITH_TORCH, TorchTensor

if WITH_TORCH:
    import torch
    import torchvision




def plot_pytorch_tensor_images(images: TorchTensor, title: str,
                               images_per_row: int) -> None:
    """Plot the images represented as PyTorch.Tensor tensors

    Parameters
    ----------
    images: The images to plot. 4D mini-batch Tensor of shape (B x C x H x W)
    title: The title of the plot
    images_per_row: Number of images in each row of the grid

    Returns
    -------
    None
    """

    if not WITH_TORCH:
        

    plt.figure()
    plt.title(title)
    plt.imshow(torchvision.utils.make_grid(images, nrow=images_per_row).permute(1, 2, 0))



from pathlib import Path

from navalmartin_mir_vision_utils import (show_pil_image,
                                          is_valid_pil_image_from_bytes_string,
                                          load_img,
                                          ImageLoadersEnumType,
                                          pil_image_to_bytes_string)


if __name__ == '__main__':

    image_path = Path("/home/alex/qi3/mir-engine/datasets/cracks_v_3_id_8/train/cracked/img_9_9.jpg")
    image = load_img(path=image_path, loader=ImageLoadersEnumType.PIL)

    show_pil_image(image=image)

    image_bytes = pil_image_to_bytes_string(image=image)
    image = is_valid_pil_image_from_bytes_string(image_byte_string=image_bytes)
    show_pil_image(image=image)
